@@ -222,6 +222,63 @@ public class RdbPushSubscriptionRegistry implements PushSubscriptionRegistry {
         return subscriptions;
     }
 
+    /**
+     * Loads the subscription identified by given ID.
+     *
+     * @param id The ID
+     * @param userId The user identifier
+     * @param contextId The context identifier
+     * @param con The connection to use
+     * @return The subscription or <code>null</code>
+     * @throws OXException If load attempt fails
+     */
+    public static PushSubscription loadById(byte[] id, int userId, int contextId, Connection con) throws OXException {
+        PreparedStatement stmt = null;
+        ResultSet rs = null;
+        try {
+            stmt = con.prepareStatement(
+                "SELECT p.token, p.client, p.transport, p.all_flag, w.topic AS prefix, e.topic FROM pns_subscription p"
+                    + " LEFT JOIN pns_subscription_topic_wildcard w ON p.id=w.id"
+                    + " LEFT JOIN pns_subscription_topic_exact e ON p.id=e.id"
+                    + " WHERE p.id=? AND p.cid=? AND p.user=?");
+            stmt.setBytes(1, id);
+            stmt.setInt(2, contextId);
+            stmt.setInt(3, userId);
+            rs = stmt.executeQuery();
+            if (false == rs.next()) {
+                return null;
+            }
+
+            List<String> topics = new LinkedList<>();
+            DefaultPushSubscription.Builder builder = DefaultPushSubscription.builder();
+            builder.contextId(contextId).userId(userId);
+            builder.token(rs.getString(1));
+            builder.client(rs.getString(2));
+            builder.transportId(rs.getString(3));
+            if (rs.getInt(4) > 0) {
+                topics.add(KnownTopic.ALL.getName());
+            }
+            do {
+                String prefix = rs.getString(6);
+                if (null != prefix) {
+                    // E.g. "ox:mail:*"
+                    topics.add(prefix + (prefix.endsWith(":") ? "*" : ":*"));
+                }
+                String topic = rs.getString(7);
+                if (null != topic) {
+                    // E.g. "ox:mail:new"
+                    topics.add(topic);
+                }
+            } while (rs.next());
+            builder.topics(topics);
+            return builder.build();
+        } catch (SQLException e) {
+            throw PushExceptionCodes.SQL_ERROR.create(e, e.getMessage());
+        } finally {
+            Databases.closeSQLStuff(rs, stmt);
+        }
+    }
+
     @Override
     public boolean hasInterestedSubscriptions(int userId, int contextId, String topic) throws OXException {
         return hasInterestedSubscriptions(null, userId, contextId, topic);
@@ -295,18 +352,15 @@ public class RdbPushSubscriptionRegistry implements PushSubscriptionRegistry {
         // Check cache
         RdbPushSubscriptionRegistryCache cache = this.cache;
         if (null != cache) {
-            MapBackedHits hits = cache.getCollectionFor(userId, contextId).getInterestedSubscriptions(client, topic);
-            return hits;
+            return cache.getCollectionFor(userId, contextId).getInterestedSubscriptions(client, topic);
         }
 
-        Map<ClientAndTransport, List<PushMatch>> subscriptions;
         Connection con = databaseService.getReadOnly(contextId);
         try {
-            subscriptions = getSubscriptions(client, userId, contextId, topic, con);
+            return getSubscriptions(client, userId, contextId, topic, con);
         } finally {
             databaseService.backReadOnly(contextId, con);
         }
-        return null == subscriptions || subscriptions.isEmpty() ? MapBackedHits.EMPTY : new MapBackedHits(subscriptions);
     }
 
     private void unregisterSubscriptions(int contextId, List<PushSubscription> subscriptions) throws OXException {
@@ -345,36 +399,29 @@ public class RdbPushSubscriptionRegistry implements PushSubscriptionRegistry {
     }
 
     /**
-     * Gets all subscriptions interested in specified topic belonging to certain users.
+     * Gets all subscriptions interested in specified topic belonging to given user.
      *
      * @param optClient The optional client to filter by
-     * @param userIds The user identifiers
+     * @param userId The user identifier
      * @param contextId The context identifier
      * @param topic The topic
      * @param con The connection to use
      * @return All subscriptions for specified affiliation
      * @throws OXException If subscriptions cannot be returned
      */
-    private Map<ClientAndTransport, List<PushMatch>> getSubscriptions(String optClient, int userId, int contextId, String topic, Connection con) throws OXException {
+    public MapBackedHits getSubscriptions(String optClient, int userId, int contextId, String topic, Connection con) throws OXException {
         if (null == con) {
-            return getInterestedSubscriptions(optClient, userId, contextId, topic).getMap();
+            return getInterestedSubscriptions(optClient, userId, contextId, topic);
         }
 
-        StringBuilder stringBuilder = new StringBuilder()
-            .append("SELECT s.id, s.user, s.token, s.client, s.transport, s.last_modified, s.all_flag, twc.topic wildcard, te.topic FROM pns_subscription s")
-            .append(" LEFT JOIN pns_subscription_topic_wildcard twc ON s.id=twc.id")
-            .append(" LEFT JOIN pns_subscription_topic_exact te ON s.id=te.id")
-            .append(" WHERE s.cid=? AND s.user=?")
-        ;
-        if (null != optClient) {
-            stringBuilder.append(" AND s.client=?");
-        }
-        stringBuilder.append(" AND ((s.all_flag=1) OR (te.topic=?) OR (? LIKE CONCAT(twc.topic, '%')));");
-
         PreparedStatement stmt = null;
         ResultSet rs = null;
         try {
-            stmt = con.prepareStatement(stringBuilder.toString());
+            stmt = con.prepareStatement(
+                "SELECT s.id, s.token, s.client, s.transport, s.last_modified, s.all_flag, twc.topic wildcard, te.topic FROM pns_subscription s" +
+                " LEFT JOIN pns_subscription_topic_wildcard twc ON s.id=twc.id" +
+                " LEFT JOIN pns_subscription_topic_exact te ON s.id=te.id" +
+                " WHERE s.cid=? AND s.user=?" + (null == optClient ? "" : " AND s.client=?") + " AND ((s.all_flag=1) OR (te.topic=?) OR (? LIKE CONCAT(twc.topic, '%')));");
             int pos = 1;
             stmt.setInt(pos++, contextId);
             stmt.setInt(pos++, userId);
@@ -386,7 +433,7 @@ public class RdbPushSubscriptionRegistry implements PushSubscriptionRegistry {
             rs = stmt.executeQuery();
 
             if (false == rs.next()) {
-                return null;
+                return MapBackedHits.EMPTY;
             }
 
             Map<ClientAndTransport, List<PushMatch>> map = new LinkedHashMap<>(6);
@@ -396,19 +443,19 @@ public class RdbPushSubscriptionRegistry implements PushSubscriptionRegistry {
 
                 // Check if not yet processed
                 if (processed.add(id)) {
-                    String token = rs.getString(3);
-                    String client = rs.getString(4);
-                    String transportId = rs.getString(5);
-                    Date lastModified = new Date(rs.getLong(6));
+                    String token = rs.getString(2);
+                    String client = rs.getString(3);
+                    String transportId = rs.getString(4);
+                    Date lastModified = new Date(rs.getLong(5));
 
                     // Determine matching topic
                     String matchingTopic;
                     {
-                        boolean all = rs.getInt(7) > 0;
+                        boolean all = rs.getInt(6) > 0;
                         if (all) {
                             matchingTopic = KnownTopic.ALL.getName();
                         } else {
-                            matchingTopic = rs.getString(8);
+                            matchingTopic = rs.getString(7);
                             if (rs.wasNull()) {
                                 matchingTopic = null;
                             } else {
@@ -423,7 +470,7 @@ public class RdbPushSubscriptionRegistry implements PushSubscriptionRegistry {
 
                             if (null == matchingTopic) { // Last reason why that row is present in result set
                                 // E.g. "ox:mail:new"
-                                matchingTopic = rs.getString(9);
+                                matchingTopic = rs.getString(8);
                             }
                         }
                     }
@@ -435,7 +482,7 @@ public class RdbPushSubscriptionRegistry implements PushSubscriptionRegistry {
                 }
             } while (rs.next());
 
-            return map;
+            return new MapBackedHits(map);
         } catch (SQLException e) {
             throw PushExceptionCodes.SQL_ERROR.create(e, e.getMessage());
         } finally {
@@ -566,7 +613,7 @@ public class RdbPushSubscriptionRegistry implements PushSubscriptionRegistry {
             stmt = con.prepareStatement(
                 "INSERT INTO pns_subscription (id,cid,user,token,client,transport,all_flag,last_modified) " +
                 "VALUES (?,?,?,?,?,?,?,?) " +
-                "ON DUPLICATE KEY UPDATE transport=?, all_flag=?, last_modified=?"
+                "ON DUPLICATE KEY UPDATE transport=?, all_flag=?, last_modified=?;"
             );
             stmt.setBytes(1, id);
             stmt.setInt(2, subscription.getContextId());
