@@ -58,6 +58,7 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.concurrent.TimeUnit;
+import org.json.JSONException;
 import org.osgi.framework.BundleContext;
 import org.osgi.framework.ServiceReference;
 import org.osgi.framework.ServiceRegistration;
@@ -71,6 +72,11 @@ import com.openexchange.config.cascade.ConfigViewFactory;
 import com.openexchange.exception.OXException;
 import com.openexchange.java.SortableConcurrentList;
 import com.openexchange.osgi.util.RankedService;
+import com.openexchange.pns.ApnsConstants;
+import com.openexchange.pns.DefaultPushSubscription;
+import com.openexchange.pns.EnabledKey;
+import com.openexchange.pns.KnownTransport;
+import com.openexchange.pns.Message;
 import com.openexchange.pns.PushExceptionCodes;
 import com.openexchange.pns.PushMatch;
 import com.openexchange.pns.PushMessageGenerator;
@@ -78,11 +84,7 @@ import com.openexchange.pns.PushMessageGeneratorRegistry;
 import com.openexchange.pns.PushNotification;
 import com.openexchange.pns.PushNotificationTransport;
 import com.openexchange.pns.PushNotifications;
-import com.openexchange.pns.DefaultPushSubscription;
-import com.openexchange.pns.EnabledKey;
-import com.openexchange.pns.KnownTransport;
 import com.openexchange.pns.PushSubscriptionRegistry;
-import com.openexchange.pns.Message;
 import com.openexchange.pns.transport.apn.ApnOptions;
 import com.openexchange.pns.transport.apn.ApnOptionsPerClient;
 import com.openexchange.pns.transport.apn.ApnOptionsProvider;
@@ -91,9 +93,9 @@ import javapns.communication.exceptions.CommunicationException;
 import javapns.communication.exceptions.KeystoreException;
 import javapns.devices.Device;
 import javapns.devices.exceptions.InvalidDeviceTokenFormatException;
-import javapns.json.JSONException;
 import javapns.notification.Payload;
 import javapns.notification.PayloadPerDevice;
+import javapns.notification.PushNotificationBigPayload;
 import javapns.notification.PushNotificationPayload;
 import javapns.notification.PushedNotification;
 import javapns.notification.PushedNotifications;
@@ -115,8 +117,6 @@ public class ApnPushNotificationTransport extends ServiceTracker<ApnOptionsProvi
 
     private static final int STATUS_INVALID_TOKEN = 8;
 
-    private static final int MAX_PAYLOAD_SIZE = 256;
-
     // ---------------------------------------------------------------------------------------------------------------
 
     private final ConfigViewFactory configViewFactory;
@@ -356,8 +356,8 @@ public class ApnPushNotificationTransport extends ServiceTracker<ApnOptionsProvi
     private void addCheckPayload(Payload payload, PushMatch match, List<PayloadPerDevice> payloads) throws OXException {
         int payloadLength = PushNotifications.getPayloadLength(payload.toString());
         // Check payload length
-        if (payloadLength > MAX_PAYLOAD_SIZE) {
-            throw PushExceptionCodes.MESSAGE_TOO_BIG.create(MAX_PAYLOAD_SIZE, payloadLength);
+        if (payloadLength > ApnsConstants.APNS_MAX_PAYLOAD_SIZE) {
+            throw PushExceptionCodes.MESSAGE_TOO_BIG.create(ApnsConstants.APNS_MAX_PAYLOAD_SIZE, payloadLength);
         }
 
         try {
@@ -365,7 +365,10 @@ public class ApnPushNotificationTransport extends ServiceTracker<ApnOptionsProvi
         } catch (InvalidDeviceTokenFormatException e) {
             LOG.warn("Invalid device token: '{}', removing from subscription store.", match.getToken(), e);
             try {
-                subscriptionRegistry.unregisterSubscription(DefaultPushSubscription.instanceFor(match));
+                boolean unregistered = subscriptionRegistry.unregisterSubscription(DefaultPushSubscription.instanceFor(match));
+                if (false == unregistered) {
+                    LOG.error("Failed to remove subscription for invalid token {}", match.getToken());
+                }
             } catch (OXException x) {
                 LOG.error("Failed to remove subscription for invalid token {}", match.getToken(), x);
             }
@@ -373,7 +376,7 @@ public class ApnPushNotificationTransport extends ServiceTracker<ApnOptionsProvi
     }
 
     private Payload toPayload(Map<String, Object> message) throws JSONException {
-        PushNotificationPayload payload = new PushNotificationPayload();
+        PushNotificationPayload payload = new PushNotificationBigPayload();
 
         Map<String, Object> source = new HashMap<>(message);
         {
@@ -428,26 +431,32 @@ public class ApnPushNotificationTransport extends ServiceTracker<ApnOptionsProvi
      * Queries the feedback service and processes the received results, removing reported tokens from the subscription store if needed.
      */
     public void queryFeedbackService() {
+        Map<String, ApnOptions> options = getAllHighestRankedApnOptions();
+        if (options.isEmpty()) {
+            return;
+        }
+
         LOG.info("Querying APNS feedback service for 'apn'...");
         long start = System.currentTimeMillis();
 
-        Map<String, ApnOptions> options = getAllHighestRankedApnOptions();
-        for (ApnOptions apnOptions : options.values()) {
+        for (Map.Entry<String, ApnOptions> entry : options.entrySet()) {
             List<Device> devices = null;
             try {
+                ApnOptions apnOptions = entry.getValue();
                 devices = Push.feedback(apnOptions.getKeystore(), apnOptions.getPassword(), apnOptions.isProduction());
             } catch (Exception e) {
                 LOG.warn("error querying feedback service", e);
             }
 
+            String clientId = entry.getKey();
             if (null != devices && !devices.isEmpty()) {
                 for (Device device : devices) {
                     LOG.debug("Got feedback for device with token: {}, last registered: {}", device.getToken(), device.getLastRegister());
                     int numRemoved = removeSubscriptions(device);
-                    LOG.info("Removed {} subscriptions for device with token: {}.", numRemoved, device.getToken());
+                    LOG.info("Removed {} subscriptions associated with client {} for device with token: {}.", numRemoved, clientId, device.getToken());
                 }
             } else {
-                LOG.info("No devices to unregister received from feedback service.");
+                LOG.info("No devices to unregister received from feedback service for client {}.", clientId);
             }
         }
 
